/***************************************************************************
 * Name:      Ma_Liste.cpp                                                 *
 * Purpose:   Code for the wxListCtrl                                      *
 * Author:    Jackie Team (jackie.team.com)                                *
 * Created:   2007-11-26                                                   *
 * Copyright: Jackie Team (www.musicorganizer.org)                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pb.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__
#include "components.h"
#include "images/up.xpm"
#include "images/down.xpm"
#include <map>
#include <wx/filename.h>

IMPLEMENT_DYNAMIC_CLASS(MaListe, wxListCtrl)


BEGIN_EVENT_TABLE(MaListe, wxListCtrl)
    EVT_LIST_ITEM_SELECTED(wxID_ANY, MaListe::onItemSelected)
    EVT_LIST_ITEM_DESELECTED(wxID_ANY, MaListe::onItemDeselected)
    EVT_KEY_DOWN(MaListe::touche)
    EVT_KEY_UP(MaListe::toucheHaut)
    EVT_LIST_COL_CLICK(wxID_ANY,MaListe::sort)
    EVT_CONTEXT_MENU(MaListe::onContextMenu)
    EVT_MENU(MusicOrganizer::ID_REFRESH, MaListe::onRefresh)
    EVT_MENU(MusicOrganizer::ID_DEL, MaListe::onDel)
    EVT_MENU(MusicOrganizer::ID_SEL, MaListe::onSel)
    EVT_MENU(ID_PLAY, MaListe::onPlay)
    EVT_MENU(ID_ADV, MaListe::onAdv)
    EVT_MENU(wxID_ANY, MaListe::onReset)
    EVT_LIST_ITEM_ACTIVATED(wxID_ANY,MaListe::onDClick)
    EVT_LIST_COL_RIGHT_CLICK(wxID_ANY,MaListe::onRColClick)
    EVT_KILL_FOCUS(MaListe::onLostFocus)
END_EVENT_TABLE()


MaListe::MaListe(wxWindow *parent, MonBook* b,MonChoicebook* c) : wxListCtrl(parent,-1,
            wxDefaultPosition, wxDefaultSize,
            wxSUNKEN_BORDER | wxLC_REPORT)/*,
        m_attr(*wxBLUE, *wxLIGHT_GREY, wxNullFont)*/
{
#ifdef __POCKETPC__
    EnableContextMenu();
#endif

    //add the column
    InsertColumn(NAME_COL, _("Name"));
    InsertColumn(TITLE_COL, _("Title"));
    InsertColumn(ARTIST_COL, _("Artist"));
    InsertColumn(ALBUM_COL, _("Album"));
    InsertColumn(YEAR_COL, _("Year"));
    InsertColumn(GENRE_COL, _("Genre"));
    InsertColumn(TRACK_COL, _("Track"));
    InsertColumn(LEN_COL, _("Length"));
    InsertColumn(DIR_COL, _("Path"));

    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(-1);
#ifdef __WXMSW__
    for(int i=0; i<=8 ; i++)
        SetColumn(i, item);//remove picture
#endif

    //Restore column width if there were saved else the default width is given
    SetColumnWidth(NAME_COL, OptionsHandler::getOption(_T("/list/columnsWidth/name"),210));
    SetColumnWidth(TITLE_COL, OptionsHandler::getOption(_T("/list/columnsWidth/title"),100));
    SetColumnWidth(ARTIST_COL, OptionsHandler::getOption(_T("/list/columnsWidth/artist"),100));
    SetColumnWidth(ALBUM_COL, OptionsHandler::getOption(_T("/list/columnsWidth/album"),100));
    SetColumnWidth(YEAR_COL, OptionsHandler::getOption(_T("/list/columnsWidth/year"),55));
    SetColumnWidth(GENRE_COL, OptionsHandler::getOption(_T("/list/columnsWidth/genre"),100));
    SetColumnWidth(TRACK_COL, OptionsHandler::getOption(_T("/list/columnsWidth/track"),55));
    SetColumnWidth(LEN_COL, OptionsHandler::getOption(_T("/list/columnsWidth/length"),55));
    SetColumnWidth(DIR_COL, OptionsHandler::getOption(_T("/list/columnsWidth/dir"),220));

    book=b;
    currentCol=-1;
    columnImageList=new wxImageList(16,16,true);
    columnImageList->Add(wxIcon(up_xpm));
    columnImageList->Add(wxIcon(down_xpm));
    SetImageList(columnImageList,wxIMAGE_LIST_SMALL);
    is_key_pressed=false;
    cb=c;
    enable=true;
    Connect(wxID_ANY, wxEVT_COMMAND_LIST_BEGIN_DRAG,wxListEventHandler(MaListe::onDragInit));
}


/* ********************************************************************************* */

void MaListe::setArbo(MonArbo* a) {
    arbo=a;
}

/* ********************************************************************************* */

void MaListe::setEnableEvent(bool e) {
    enable=e;
}

/* ********************************************************************************* */

long MaListe::getSelection(int nextSel) {
    return GetNextItem(nextSel, wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED);
}

/* ********************************************************************************* */

wxString MaListe::getRowText(long  index, int col) {
    if (index >= 0) {
        wxListItem i;
        i.SetColumn(col);
        i.SetId(index);
        i.SetMask(wxLIST_MASK_TEXT);
        GetItem(i);
        return i.GetText();
    } else
        return wxEmptyString;
}

/* ********************************************************************************* */

wxString MaListe::getSelRowText(int col, int nextSel) {
    return getRowText(getSelection(nextSel), col);
}


/* ********************************************************************************* */

void MaListe::onItemSelected(wxListEvent& event) {
    if(!enable)
        return;
    sendMenuMsg(0);//all actions can be done
    cb->selectItem(event.GetIndex(),this);
}

/* ********************************************************************************* */

void MaListe::onItemDeselected(wxListEvent& event) {
    if(!enable)//if vetoed
        return;
    cb->selectItem(event.GetIndex(),this,false);
    if ((GetSelectedItemCount() ) == 0) {
        sendMenuMsg(2);//refresh plus select all can be done now
        //clean combobox
        book->getComboAlbum()->clearAll();
        book->getComboAlbum()->SetValue(wxEmptyString);
        book->getComboTitre()->clearAll();
        book->getComboTitre()->SetValue(wxEmptyString);
        book->getComboInterprete()->clearAll();
        book->getComboInterprete()->SetValue(wxEmptyString);
        book->getComboAnne()->SetValue(wxEmptyString);
        book->getComboGenre()->SetValue(wxEmptyString);
        book->getComboPiste()->clearAll();
        book->getComboPiste()->SetValue(wxEmptyString);
        book->getComboAlbum2()->clearAll();
        book->getComboAlbum2()->SetValue(wxEmptyString);
        book->getComboTitre2()->clearAll();
        book->getComboTitre2()->SetValue(wxEmptyString);
        book->getComboInterprete2()->clearAll();
        book->getComboInterprete2()->SetValue(wxEmptyString);
        book->getComboAnne2()->SetValue(wxEmptyString);
        book->getComboGenre2()->SetValue(wxEmptyString);
        book->getComboPiste2()->clearAll();
        book->getComboPiste2()->SetValue(wxEmptyString);
    }
}

/* ********************************************************************************* */

void MaListe::toucheHaut(wxKeyEvent& event) {
    is_key_pressed=0;

    event.Skip();
}


/* ********************************************************************************* */


void MaListe::touche(wxKeyEvent& event) {
    if(is_key_pressed == false) { //if it's not vetoed
        if(event.GetModifiers() == wxMOD_CMD ) {
            if(
                (wxUSE_UNICODE == 1 &&
                 (event.GetUnicodeKey() == wxChar('A') || event.GetUnicodeKey() == wxChar('A'))
                )
                ||
                (event.GetKeyCode() == (int)'A' || event.GetKeyCode() == (int)'A')
            ) { //[CTRL]+[A]
                for(long i=0; i<GetItemCount() ; i++) { //for each of the whole list
                    SetItemState(i,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);//select it
                }
            }
        } else if(event.GetModifiers() == wxMOD_NONE ) {
            if(event.GetKeyCode() == WXK_DELETE) { //[DEL]
                cb->delCurrent();
            } else if(event.GetKeyCode() == WXK_F5) { //[F5]
                is_key_pressed = true;
                arbo->refresh();
            }
        }
    }
    event.Skip();
}


/* ********************************************************************************* */

void MaListe::sort(wxListEvent& event) {
    if(currentCol==event.GetColumn())
        desc=!desc;
    else
        desc=false;

    cb->sortLists(currentCol,event.GetColumn(),desc,this);
}

/* ********************************************************************************* */

void MaListe::setCurCol(int col) {
    currentCol=col;
}

/* ********************************************************************************* */

void MaListe::removeColumnImage() {
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(-1);//none image
#ifdef __WXMSW__
    SetColumn(currentCol, item);//delete picture
#endif
}

/* ********************************************************************************* */

void MaListe::onContextMenu(wxContextMenuEvent& event) {
    wxPoint point = event.GetPosition();
    // If from keyboard
    if (point.x == -1 && point.y == -1) {
        wxSize size = GetSize();
        point.x = size.x / 2;
        point.y = size.y / 2;
    } else {
        point = ScreenToClient(point);
    }
    wxMenu menu;

    menu.Append(MusicOrganizer::ID_REFRESH, _("&Refresh\tF5"));
    menu.Append(MusicOrganizer::ID_DEL, _("&Delete\tDel"));
    menu.Append(MusicOrganizer::ID_SEL, _("&Select all\tCtrl-A"));
    menu.Append(ID_PLAY, _("&Play"));
    //disable for 1.5.1
    //menu.Append(ID_ADV, _("&Advanced Properties"));

    if(GetItemCount()==0)
        menu.Enable(MusicOrganizer::ID_SEL,false);
    if(GetSelectedItemCount()==0) {
        menu.Enable(MusicOrganizer::ID_DEL,false);
        menu.Enable(ID_PLAY,false);
    }
    if(GetSelectedItemCount()!=1)
        menu.Enable(ID_ADV,false);

    //show menu
    PopupMenu(&menu, point.x, point.y);


}

/* ********************************************************************************* */

/**
 * This is done in order to keep the selection style
 * when the wxListCtrl lost the focus
 */
void MaListe::onLostFocus(wxFocusEvent& event) {
    return;
}

/* ********************************************************************************* */

void MaListe::onRefresh(wxCommandEvent& WXUNUSED(event)) {
    if(is_key_pressed == false) { //to prevent multiple refresh at the same time
        arbo->refresh();
        is_key_pressed=true;
    }
}

/* ********************************************************************************* */

void MaListe::onDel(wxCommandEvent& WXUNUSED(event)) {
    cb->delCurrent();
}

/* ********************************************************************************* */

void MaListe::onSel(wxCommandEvent& WXUNUSED(event)) {
    for(long i=0; i<GetItemCount() ; i++) { //select all items
        SetItemState(i,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
    }
}

/* ********************************************************************************* */

void MaListe::sendMenuMsg(int mode) {
    wxCommandEvent event( wxEVT_COMMAND_MENU_SELECTED, MusicOrganizer::ID_MSG);
    event.SetInt(mode);
    wxPostEvent(GetGrandParent(), event );
}

/* ********************************************************************************* */

void MaListe::unblockKeyEvent(bool b) {
    is_key_pressed=!b;
}

/* ********************************************************************************* */

void MaListe::onDClick(wxListEvent& e) {
    playCurrent();
}

/* ********************************************************************************* */

void MaListe::onPlay(wxCommandEvent& WXUNUSED(event)) {
    playCurrent();
}

/* ********************************************************************************* */

void MaListe::playCurrent() {
    long sel = getSelection(-1);
    wxMimeTypesManager mime;
    //make the playist using the m3u format
    wxString args(wxEmptyString),sep=wxString::Format(_T("%c"),wxFileName::GetPathSeparator());
    while(sel !=-1) { //for each selected files
        args+=getRowText(sel,DIR_COL)+sep+getRowText(sel,NAME_COL);
        sel = getSelection(sel);
        args+=_T("\n");
    }
    wxString s=wxFileName::CreateTempFileName(_T("cur"));//get a tempory file name
    wxRemoveFile(s);//the name is not totaly good and as it's automaticly created we delete it
    s+=wxString(_T(".m3u"));//correct file name

    wxFile f(s,wxFile::write);
    f.Write(args);//write playlist content
    f.Close();

    //get player and launch it
    wxString player=OptionsHandler::getOption(_T("/list/player"),wxString(wxT("-1")));
    if(player==_T("-1"))//use the default player
        wxExecute(mime.GetFileTypeFromExtension(_T("mp3"))->GetOpenCommand(s));
    else //use custom player
        wxExecute(player+_T(" ")+s);

}

/* ********************************************************************************* */

void MaListe::onRColClick(wxListEvent& e) {
    //Right Click on a column header
    wxPoint point = wxGetMousePosition();
    point = ScreenToClient(point);
    wxMenu menu;

    menu.Append(e.GetColumn(), _("&Reset column"));

    PopupMenu(&menu, point.x, point.y);

}

/* ********************************************************************************* */

void MaListe::onReset(wxCommandEvent& e) {
    int size=-1;
    switch(e.GetId()) { //reset default width
    case NAME_COL:
        size=210;
        break;
    case TITLE_COL:
        size=100;
        break;
    case ARTIST_COL:
        size=100;
        break;
    case ALBUM_COL:
        size=100;
        break;
    case YEAR_COL:
        size=55;
        break;
    case GENRE_COL:
        size=100;
        break;
    case TRACK_COL:
        size=55;
        break;
    case LEN_COL:
        size=55;
        break;
    case DIR_COL:
        size=220;
        break;
    }
    if(size!=-1)
        SetColumnWidth(e.GetId(),size);

}

/* ********************************************************************************* */

void MaListe::recolor() {
    if (GetItemCount() == 0)
        return ;
    wxColour blanc      = wxColour(255,255,255);
    wxColour grisClair  = wxColour(240,240,240);
    wxColour secColor   = wxColour(206,235,253);
    wxColour secColor2  = wxColour(230,248,254);
    int numMusicInFolder = 0;

    wxColour* currentColor1 = &blanc;
    wxColour* currentColor2 = &grisClair;

    SetItemBackgroundColour(0,*currentColor1);//The first item always have a white background
    for (int index = 1; index < GetItemCount(); index++) {
        if (getRowText(index,MaListe::DIR_COL) != getRowText(index-1,MaListe::DIR_COL)) { //new folder
            numMusicInFolder = 0;
            currentColor1 = (currentColor1 == &blanc) ? &secColor : &blanc;
            currentColor2 = (currentColor2 == &grisClair) ? &secColor2 : &grisClair;
        } else {
            numMusicInFolder++;
        }
        if (numMusicInFolder%2 == 0) {
            SetItemBackgroundColour(index,*currentColor1);
        } else {
            SetItemBackgroundColour(index,*currentColor2);
        }
    }
}

/* ********************************************************************************* */

void MaListe::saveColumnsWidth() {
    OptionsHandler::setOption(_T("/list/columnsWidth/name"),GetColumnWidth(NAME_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/title"),GetColumnWidth(TITLE_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/artist"),GetColumnWidth(ARTIST_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/album"),GetColumnWidth(ALBUM_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/year"),GetColumnWidth(YEAR_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/genre"),GetColumnWidth(GENRE_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/length"),GetColumnWidth(LEN_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/track"),GetColumnWidth(TRACK_COL));
    OptionsHandler::setOption(_T("/list/columnsWidth/dir"),GetColumnWidth(DIR_COL));
}

/* ********************************************************************************* */

void MaListe::onDragInit(wxListEvent& e) {
    wxString text=wxEmptyString;
    long nextSel=-1;

    //create the string for the drag data. It's all files separated by "\n"
    for (int i=0 ; i<GetSelectedItemCount() ; i++) {
        text+=getSelRowText(DIR_COL,nextSel)+wxString::Format(_T("%c"),wxFileName::GetPathSeparator())+getSelRowText(NAME_COL,nextSel);
        if(i!=GetSelectedItemCount()-1)
            text+=_T("\n");
        nextSel=getSelection(nextSel);
    }

    wxTextDataObject tdo(text);
    wxDropSource tds(this);
    tds.SetData(tdo);//attach the files list
    tds.DoDragDrop(wxDrag_AllowMove);//Begin drag


}

void MaListe::onAdv(wxCommandEvent& WXUNUSED(event)) {
    apd = new AdvPropertiesDialog(cb,this);

}



/* *****************************************************************
   ***************************************************************** */
ListCtrlPanel::ListCtrlPanel(wxWindow *parent,MonBook* b):wxPanel(parent) {
    wxBoxSizer* sizer=new wxBoxSizer(wxVERTICAL);
    cb=new MonChoicebook(this,b);

    sizer->Add(cb,1,wxEXPAND|wxALL);
    SetSizer(sizer);
}

MonChoicebook* ListCtrlPanel::getCb() {
    return cb;
}


