/***************************************************************************
 *   Copyright (C) 2007 by Jackie Team                                     *
 *   jackie.team@gmail.com                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//taglib includes
#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/id3v1tag.h>
#include <taglib/id3v2tag.h>
#include <taglib/mpegfile.h>
#ifdef USE_MP4
#include <taglib/mp4file.h>
#endif

#include <wx/dynarray.h>
#include "components.h"
#include <iostream>

BEGIN_EVENT_TABLE(MonChoicebook, wxChoicebook)
    EVT_CHOICEBOOK_PAGE_CHANGED(wxID_ANY, MonChoicebook::onChanged)
END_EVENT_TABLE()

MonChoicebook::MonChoicebook(wxWindow* parent,MonBook* b):wxChoicebook(parent,wxID_ANY) {
    book=b;
    wxPanel* panelPage1 = new wxPanel(this);
    wxPanel* panelPage2 = new wxPanel(this);
    wxPanel* panelPage3 = new wxPanel(this);
    wxPanel* panelPage4 = new wxPanel(this);
    mlisteV1=new MaListe(panelPage1,b,this);
    mlisteV2=new MaListe(panelPage2,b,this);
    mlisteFlac=new MaListe(panelPage3,b,this);
    mlisteOgg=new MaListe(panelPage4,b,this);
    AddPage(panelPage2,_T("MP3 ID3 V2"),true);
    AddPage(panelPage1,_T("MP3 ID3 V1"),false);
    AddPage(panelPage3,_T("Flac"),false);
    AddPage(panelPage4,_T("Ogg"),false);

    wxBoxSizer* sizerV1=new wxBoxSizer(wxHORIZONTAL),*sizerV2=new wxBoxSizer(wxHORIZONTAL),
    *sizerFlac=new wxBoxSizer(wxHORIZONTAL),*sizerOgg=new wxBoxSizer(wxHORIZONTAL);

    sizerV1->Add(mlisteV1,1,wxEXPAND|wxALL);
    panelPage1->SetSizer(sizerV1);
    sizerV2->Add(mlisteV2,1,wxEXPAND|wxALL);
    panelPage2->SetSizer(sizerV2);
    sizerFlac->Add(mlisteFlac,1,wxEXPAND|wxALL);
    panelPage3->SetSizer(sizerFlac);
    sizerOgg->Add(mlisteOgg,1,wxEXPAND|wxALL);
    panelPage4->SetSizer(sizerOgg);
#ifdef USE_MP4
    wxPanel* panelPage5 = new wxPanel(this);
    wxBoxSizer *sizerMp4=new wxBoxSizer(wxHORIZONTAL);
    mlisteMp4=new MaListe(panelPage5,b,this);
    AddPage(panelPage5,_T("MP4"),false);
    sizerMp4->Add(mlisteMp4,1,wxEXPAND|wxALL);
    panelPage5->SetSizer(sizerMp4);
#endif
}

/* ****************************************************************************** */

MaListe* MonChoicebook::getMListeFlac() {
    return mlisteFlac;
}

/* ****************************************************************************** */

MaListe* MonChoicebook::getMListeOgg() {
    return mlisteOgg;
}

/* ****************************************************************************** */

MaListe* MonChoicebook::getMListeV1() {
    return mlisteV1;
}

/* ****************************************************************************** */

MaListe* MonChoicebook::getMListeV2() {
    return mlisteV2;
}

/* ****************************************************************************** */

#ifdef USE_MP4
MaListe* MonChoicebook::getMListeMp4() {
    return mlisteMp4;
}

/* ****************************************************************************** */
#endif

MaListe* MonChoicebook::getCurrentList() {
    if (GetSelection() == 0) return mlisteV2;
    else if (GetSelection() == 1) return mlisteV1;
    else if (GetSelection() == 2) return mlisteFlac;
    else if (GetSelection() == 3) return mlisteOgg;
#ifdef USE_MP4
    else if (GetSelection() == 4) return mlisteMp4;
#endif
    return NULL;
}

/* ****************************************************************************** */

MaListe* MonChoicebook::getOtherList() {
    if (GetSelection() == 2) return NULL;
    if (GetSelection() == 0) return mlisteV1;
    else return mlisteV2;
}

/* ****************************************************************************** */

void MonChoicebook::selectItem(long index,MaListe* who,bool sel) {
    if(getType()==_T("mp3")) {
        wxListItem current;
        //retrieve columns content of the selected item
        wxString t,i,a,p,tv1,iv1,av1,pv1;
        current.m_itemId = index;
        current.SetMask(wxLIST_MASK_TEXT);
        //ID3V2 tags
        current.SetColumn(MaListe::TITLE_COL);
        mlisteV2->GetItem(current);
        t=current.GetText();
        current.SetColumn(MaListe::ARTIST_COL);
        mlisteV2->GetItem(current);
        i=current.GetText();
        current.SetColumn(MaListe::ALBUM_COL);
        mlisteV2->GetItem(current);
        a=current.GetText();

        //ID3V1 tags
        current.SetColumn(MaListe::TITLE_COL);
        mlisteV1->GetItem(current);
        tv1=current.GetText();
        current.SetColumn(MaListe::ARTIST_COL);
        mlisteV1->GetItem(current);
        iv1=current.GetText();
        current.SetColumn(MaListe::ALBUM_COL);
        mlisteV1->GetItem(current);
        av1=current.GetText();


        if(sel) { //select item
            if(who==mlisteV1) {
                mlisteV2->setEnableEvent(false);
                mlisteV2->SetItemState(index,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                mlisteV2->setEnableEvent(true);
            } else {
                mlisteV1->setEnableEvent(false);
                mlisteV1->SetItemState(index,wxLIST_STATE_SELECTED,wxLIST_STATE_SELECTED);
                mlisteV1->setEnableEvent(true);
            }

            //add not empty tags to the combobox
            //id3V2
            if (t != wxEmptyString)
                book->getComboTitre()->addComboBoxItem(t);

            if (i != wxEmptyString)
                book->getComboInterprete()->addComboBoxItem(i);

            if (a != wxEmptyString)
                book->getComboAlbum()->addComboBoxItem(a);

            //id3V1
            if (tv1 != wxEmptyString)
                book->getComboTitre2()->addComboBoxItem(tv1);

            if (iv1 != wxEmptyString)
                book->getComboInterprete2()->addComboBoxItem(iv1);

            if (av1 != wxEmptyString)
                book->getComboAlbum2()->addComboBoxItem(av1);
        } else { //deselect item
            if(who==mlisteV1) {
                mlisteV2->setEnableEvent(false);
                mlisteV2->SetItemState(index,0,wxLIST_STATE_SELECTED);
                mlisteV2->setEnableEvent(true);
            } else {
                mlisteV1->setEnableEvent(false);
                mlisteV1->SetItemState(index,0,wxLIST_STATE_SELECTED);
                mlisteV1->setEnableEvent(true);
            }

            //remove not empty tags from the combobox
            //id3v2

            if (t != wxEmptyString)
                book->getComboTitre()->removeComboBoxItem(t);

            if (i != wxEmptyString)
                book->getComboInterprete()->removeComboBoxItem(i);

            if (a != wxEmptyString)
                book->getComboAlbum()->removeComboBoxItem(a);

            //id3v1
            if (tv1 != wxEmptyString)
                book->getComboTitre2()->removeComboBoxItem(tv1);

            if (iv1 != wxEmptyString)
                book->getComboInterprete2()->removeComboBoxItem(iv1);

            if (av1 != wxEmptyString)
                book->getComboAlbum2()->removeComboBoxItem(av1);
        }
#ifdef USE_MP4
    } else if(getType()==_T("flac") || getType()==_T("ogg") || getType()==_T("mp4")) {
#else
    } else if(getType()==_T("flac") || getType()==_T("ogg")) {
#endif
        MaListe* ml=NULL;
        if(getType()==_T("flac")) ml=mlisteFlac;
        else if(getType()==_T("ogg")) ml=mlisteOgg;
#ifdef USE_MP4
        else if(getType()==_T("mp4")) ml=mlisteMp4;
#endif

        wxListItem current;
        //retrieve columns content of the selected item
        wxString t,i,a,p;
        current.m_itemId = index;
        current.SetMask(wxLIST_MASK_TEXT);
        //flac or ogg tags
        current.SetColumn(MaListe::TITLE_COL);
        ml->GetItem(current);
        t=current.GetText();
        current.SetColumn(MaListe::ARTIST_COL);
        ml->GetItem(current);
        i=current.GetText();
        current.SetColumn(MaListe::ALBUM_COL);
        ml->GetItem(current);
        a=current.GetText();


        if(sel) { //select item

            //add not empty tags to the combobox
            //id3V2
            if (t != wxEmptyString)
                book->getComboTitre2()->addComboBoxItem(t);

            if (i != wxEmptyString)
                book->getComboInterprete2()->addComboBoxItem(i);

            if (a != wxEmptyString)
                book->getComboAlbum2()->addComboBoxItem(a);

        } else { //deselect item

            //remove not empty tags from the combobox
            //id3v2
            if (t != wxEmptyString)
                book->getComboTitre2()->removeComboBoxItem(t);

            if (i != wxEmptyString)
                book->getComboInterprete2()->removeComboBoxItem(i);

            if (a != wxEmptyString)
                book->getComboAlbum2()->removeComboBoxItem(a);
        }
    }
}

/* ****************************************************************************** */

void MonChoicebook::delCurrent() {
    long ancienneSel = mlisteV1->getSelection(-1);
    while(ancienneSel != -1) { //for each selected files
        mlisteV1->SetItemState(ancienneSel,0,wxLIST_STATE_SELECTED);
        mlisteV1->DeleteItem(ancienneSel);
        mlisteV2->DeleteItem(ancienneSel);

        ancienneSel = mlisteV1->getSelection(ancienneSel - 1);
    }
    if(mlisteV1->GetItemCount()==0)//if there is no more item in the list
        mlisteV1->sendMenuMsg(-1);//update menus
    mlisteV1->recolor();
    mlisteV2->recolor();
}

/* ****************************************************************************** */
bool MonChoicebook::readAndShowInfos(wxString f,int index) {
    wxColour blanc = wxColour(255,255,255);
    wxColour grisClair = wxColour(240,240,240);
    wxColour secColor = wxColour(206,235,253);
    wxColour secColor2 = wxColour(230,248,254);

    MusicFormater musicFormater(f, this);
    MusicFormat* musicFormat = musicFormater.getFormat(0);
    MaListe* list = musicFormater.getList(0);

    wxString name = musicFormater.getFileName();
    wxString directory = musicFormater.getFilePath();
    wxString ext = musicFormater.getFileExtension();

    int j = (ext == _T("mp3")) ? 2 : 1; //numbers of lists to update

    for (int i = 0; i < j; i++) { //two lists need update with mp3
        if (i == 1 && ext == _T("mp3")) {
            //ID3v2 update
            musicFormat = musicFormater.getFormat(1);
            list = musicFormater.getList(1);
        }
        wxString artist, title, album, genre, year, track, duree;
        if(musicFormat->exists()) {
            track  = musicFormat->getTrack();
            year   = musicFormat->getYear();
            artist = musicFormat->getArtist();
            title  = musicFormat->getTitle();
            album  = musicFormat->getAlbum();
            genre  = musicFormat->getGenre();
        }
        duree = musicFormat->getLength();
        //clean tags
        track.Replace(_T("\0"),wxEmptyString);
        year.Replace(_T("\0"),wxEmptyString);
        artist.Replace(_T("\0"),wxEmptyString);
        title.Replace(_T("\0"),wxEmptyString);
        album.Replace(_T("\0"),wxEmptyString);
        genre.Replace(_T("\0"),wxEmptyString);
        duree.Replace(_T("\0"),wxEmptyString);

        //insert the new item and fill the columns
        list->InsertItem(index,name);
#ifdef __WXMSW__
        list->SetItemImage(index,-1);
#endif
        list->SetItemData(index,index);
        list->SetItem(index,MaListe::TITLE_COL,title);
        list->SetItem(index,MaListe::ARTIST_COL,artist);
        list->SetItem(index,MaListe::ALBUM_COL,album);
        list->SetItem(index,MaListe::YEAR_COL,year);
        list->SetItem(index,MaListe::GENRE_COL,genre);
        list->SetItem(index,MaListe::TRACK_COL,track);
        list->SetItem(index,MaListe::LEN_COL,duree);
        list->SetItem(index,MaListe::DIR_COL,directory);

        //define the background colour of the item
        if (index == 0) { //first item always have a white background
            list->SetItemBackgroundColour(index,blanc);
        } else if (list->getRowText(index,MaListe::DIR_COL) == list->getRowText(index-1,MaListe::DIR_COL)) { //same folder as before
            if (list->GetItemBackgroundColour(index-1) == blanc) {
                list->SetItemBackgroundColour(index,grisClair);
            } else if (list->GetItemBackgroundColour(index-1) == grisClair) {
                list->SetItemBackgroundColour(index,blanc);
            } else if (list->GetItemBackgroundColour(index-1) == secColor) {
                list->SetItemBackgroundColour(index,secColor2);
            } else {
                list->SetItemBackgroundColour(index,secColor);
            }
        } else { //new folder
            if (list->GetItemBackgroundColour(index-1) == blanc || list->GetItemBackgroundColour(index-1) == grisClair) {
                list->SetItemBackgroundColour(index,secColor);
            } else {
                list->SetItemBackgroundColour(index,blanc);
            }
        }
        delete musicFormat;
    }
    return true;
}

/* ****************************************************************************** */

void MonChoicebook::sortLists(int& currentCol,int oldCol,bool desc,MaListe* who) {
    MaListe* other;
    bool is_mp3=(who==mlisteV1 || who==mlisteV2);
    if(who==mlisteV1) other=mlisteV2;
    else other=mlisteV1;

    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(-1);
    who->SetColumn(currentCol, item);//delete picture
    if(is_mp3) {
        other->SetColumn(currentCol, item);//delete picture
        other->setCurCol(currentCol);
    }
    currentCol = oldCol;
    if(desc)
        item.SetImage(1);//down
    else
        item.SetImage(0);//up
    who->SetColumn(currentCol, item);
    if(is_mp3)
        other->SetColumn(currentCol, item);

    //the array which will contain all tags
    wxArrayString nom;
    wxArrayString titre;
    wxArrayString artiste;
    wxArrayString album;
    wxArrayString annee;
    wxArrayString genre;
    wxArrayString piste;
    wxArrayString duree;
    wxArrayString othertmp[6];
    wxArrayString chemin;
    ArrayOfInts states;

    //std::multimap are used because it sort automaticly and fastly data
    std::multimap<wxString, int>::iterator it;
    std::multimap<wxString, int>::reverse_iterator rit;
    std::multimap<wxString, int> myMap;

    bool sortByFolder = OptionsHandler::getOption(_T("/list/sort_by_dir"),true);
    for (long i=0; i<who->GetItemCount(); i++) { //for each item
        nom.Add(who->getRowText(i,MaListe::NAME_COL));
        titre.Add(who->getRowText(i,MaListe::TITLE_COL));
        artiste.Add(who->getRowText(i,MaListe::ARTIST_COL));
        album.Add(who->getRowText(i,MaListe::ALBUM_COL));
        annee.Add(who->getRowText(i,MaListe::YEAR_COL));
        genre.Add(who->getRowText(i,MaListe::GENRE_COL));
        piste.Add(who->getRowText(i,MaListe::TRACK_COL));
        duree.Add(who->getRowText(i,MaListe::LEN_COL));
        chemin.Add(who->getRowText(i,MaListe::DIR_COL));
        if(is_mp3) {
            //retreive tags of unselected list
            for(int k=0; k<6 ; k++)
                othertmp[k].Add(other->getRowText(i,k+1));
        }

        long val;
        if(sortByFolder && currentCol != MaListe::DIR_COL) {
            if(currentCol == MaListe::TRACK_COL || currentCol==MaListe::YEAR_COL) { //fix number sorting
                who->getRowText(i,currentCol).ToLong(&val);
                myMap.insert(std::pair<wxString, int>(who->getRowText(i,MaListe::DIR_COL) + wxString::Format(_T("%04d"),val),i));
            } else
                myMap.insert(std::pair<wxString, int>(who->getRowText(i,MaListe::DIR_COL) + who->getRowText(i,currentCol),i));
        } else {
            if(currentCol == MaListe::TRACK_COL || currentCol==MaListe::YEAR_COL) { //fix number sorting
                who->getRowText(i,currentCol).ToLong(&val);
                myMap.insert(std::pair<wxString, int>(wxString::Format(_T("%04d"),val),i));
            } else
                myMap.insert(std::pair<wxString, int>(who->getRowText(i,currentCol),i));
        }
    }
    long i=0;
    //refill the lists
    if(!desc) { //increasing order
        for ( it=myMap.begin(); it != myMap.end(); it++ ) { //for each item
            states.Add(who->GetItemState(it->second,wxLIST_STATE_SELECTED));
            who->SetItem(i,MaListe::NAME_COL,nom.Item(it->second));
            who->SetItem(i,MaListe::TITLE_COL,titre.Item(it->second));
            who->SetItem(i,MaListe::ARTIST_COL,artiste.Item(it->second));
            who->SetItem(i,MaListe::ALBUM_COL,album.Item(it->second));
            who->SetItem(i,MaListe::YEAR_COL,annee.Item(it->second));
            who->SetItem(i,MaListe::GENRE_COL,genre.Item(it->second));
            who->SetItem(i,MaListe::TRACK_COL,piste.Item(it->second));
            who->SetItem(i,MaListe::LEN_COL,duree.Item(it->second));
            who->SetItem(i,MaListe::DIR_COL,chemin.Item(it->second));
            if(is_mp3) {
                other->SetItem(i,MaListe::LEN_COL,duree.Item(it->second));
                other->SetItem(i,MaListe::NAME_COL,nom.Item(it->second));
                other->SetItem(i,MaListe::DIR_COL,chemin.Item(it->second));
                for(int k=0; k<6 ; k++)
                    other->SetItem(i,k+1,othertmp[k][it->second]);
            }
            i++;
        }
    } else { //decreasing order
        for ( rit=myMap.rbegin(); rit != myMap.rend(); rit++ ) { //for each item
            states.Add(who->GetItemState(rit->second,wxLIST_STATE_SELECTED));
            who->SetItem(i,MaListe::NAME_COL,nom.Item(rit->second));
            who->SetItem(i,MaListe::TITLE_COL,titre.Item(rit->second));
            who->SetItem(i,MaListe::ARTIST_COL,artiste.Item(rit->second));
            who->SetItem(i,MaListe::ALBUM_COL,album.Item(rit->second));
            who->SetItem(i,MaListe::YEAR_COL,annee.Item(rit->second));
            who->SetItem(i,MaListe::GENRE_COL,genre.Item(rit->second));
            who->SetItem(i,MaListe::TRACK_COL,piste.Item(rit->second));
            who->SetItem(i,MaListe::LEN_COL,duree.Item(rit->second));
            who->SetItem(i,MaListe::DIR_COL,chemin.Item(rit->second));
            if(is_mp3) {
                other->SetItem(i,MaListe::NAME_COL,nom.Item(rit->second));
                other->SetItem(i,MaListe::LEN_COL,duree.Item(rit->second));
                other->SetItem(i,MaListe::DIR_COL,chemin.Item(rit->second));
                for(int k=0; k<6 ; k++)
                    other->SetItem(i,k+1,othertmp[k][rit->second]);
            }
            i++;
        }
    }
    //Recoloration
    who->recolor();
    if(is_mp3)
        other->recolor();
    //Reselection
    for (i=0; i<who->GetItemCount(); i++) {
        who->SetItemState(i,states[i],wxLIST_STATE_SELECTED);
        if(is_mp3) {
            other->setEnableEvent(false);
            other->SetItemState(i,states[i],wxLIST_STATE_SELECTED);
            other->setEnableEvent(true);
        }
    }
}

/* ****************************************************************************** */

void MonChoicebook::onChanged(wxChoicebookEvent& e) {
    //the displayed list have been changed
    wxRect rectV1;
    wxRect rectV2;
    //scroll the new displayed list
    if(GetSelection()==0) { //ID3V2 list is displayed now
        book->mp3Selected(true);//enable special functionnality for mp3s
#ifdef __WXMSW__
        rectV1 = mlisteV1->GetViewRect();
        rectV2 = mlisteV2->GetViewRect();
        //use correct function, but only available for this platform (sick...)
        mlisteV2->ScrollList(0-rectV1.GetX()+rectV2.GetX(),0-rectV1.GetY()+rectV2.GetY());
#else
        int y = mlisteV1->GetScrollPos(wxVERTICAL);
        int x = mlisteV1->GetScrollPos(wxHORIZONTAL);
        wxSize s=mlisteV2->GetSize();
        mlisteV2->SetScrollPos(wxVERTICAL, y);
        mlisteV2->SetScrollPos(wxHORIZONTAL, x);
        //an ugly hack to force scrollbar and list to be redrawn
        mlisteV2->SetSize(s.x-1,s.y-1);
        mlisteV2->SetSize(s);
#endif
        book->SetPageText(3,_T("ID3V2->ID3V1"));
        for(int i=0 ; i<mlisteV1->GetColumnCount() ; i++) {
            mlisteV2->SetColumnWidth(i,mlisteV1->GetColumnWidth(i));
        }
    } else if(GetSelection()==1) { //ID3V1 list is displayed now
        book->mp3Selected(true);//enable special functionnality for mp3s
#ifdef __WXMSW__
        rectV1 = mlisteV1->GetViewRect();
        rectV2 = mlisteV2->GetViewRect();
        mlisteV1->ScrollList(0-rectV2.GetX()+rectV1.GetX(),0-rectV2.GetY()+rectV1.GetY());
#else
        mlisteV1->SetScrollPos(wxVERTICAL, mlisteV2->GetScrollPos(wxVERTICAL));
        mlisteV1->SetScrollPos(wxHORIZONTAL, mlisteV2->GetScrollPos(wxHORIZONTAL));
        wxSize s=mlisteV1->GetSize();
        mlisteV1->SetSize(s.x-1,s.y-1);
        mlisteV1->SetSize(s);
#endif
        book->SetPageText(3,_T("ID3V1->ID3V2"));
        for(int i=0 ; i<mlisteV2->GetColumnCount() ; i++) {
            mlisteV1->SetColumnWidth(i,mlisteV2->GetColumnWidth(i));
        }
    } else if(GetSelection()==2 || GetSelection()==3 || GetSelection()==4) { //Flac or ogg list is displayed now
        book->mp3Selected(false);//disable special functionnality for mp3s
    }
}

/* ****************************************************************************** */

bool MonChoicebook::isID3V2() {
    if(GetSelection() == 0) return true;
    else return false;
}

/* ****************************************************************************** */

void MonChoicebook::fill_combo() {
    if(getType()==_T("mp3")) {
        if (mlisteV2->GetSelectedItemCount() == 1) { //If only on row is selected
            book->getComboAlbum()->SetValue(mlisteV2->getSelRowText(MaListe::ALBUM_COL,-1));
            book->getComboTitre()->SetValue(mlisteV2->getSelRowText(MaListe::TITLE_COL,-1));
            book->getComboInterprete()->SetValue(mlisteV2->getSelRowText(MaListe::ARTIST_COL,-1));
            book->getComboAnne()->SetValue(mlisteV2->getSelRowText(MaListe::YEAR_COL,-1));
            book->getComboGenre()->SetValue(mlisteV2->getSelRowText(MaListe::GENRE_COL,-1));
            book->getComboPiste()->SetValue(mlisteV2->getSelRowText(MaListe::TRACK_COL,-1));
            book->getComboAlbum2()->SetValue(mlisteV1->getSelRowText(MaListe::ALBUM_COL,-1));
            book->getComboTitre2()->SetValue(mlisteV1->getSelRowText(MaListe::TITLE_COL,-1));
            book->getComboInterprete2()->SetValue(mlisteV1->getSelRowText(MaListe::ARTIST_COL,-1));
            book->getComboAnne2()->SetValue(mlisteV1->getSelRowText(MaListe::YEAR_COL,-1));
            book->getComboGenre2()->SetValue(mlisteV1->getSelRowText(MaListe::GENRE_COL,-1));
            book->getComboPiste2()->SetValue(mlisteV1->getSelRowText(MaListe::TRACK_COL,-1));
        } else if(mlisteV2->GetSelectedItemCount() != 0) {
            //Inialize combobox
            book->getComboAlbum()->SetValue(_("<Album>"));
            book->getComboTitre()->SetValue(_("<Title>"));
            book->getComboInterprete()->SetValue(_("<Artist>"));
            book->getComboAnne()->SetValue(_("<Year>"));
            book->getComboPiste()->SetValue(_("<Track>"));
            book->getComboGenre()->SetValue(_("<Genre>"));
            book->getComboAlbum2()->SetValue(_("<Album>"));
            book->getComboTitre2()->SetValue(_("<Title>"));
            book->getComboInterprete2()->SetValue(_("<Artist>"));
            book->getComboAnne2()->SetValue(_("<Year>"));
            book->getComboPiste2()->SetValue(_("<Track>"));
            book->getComboGenre2()->SetValue(_("<Genre>"));
        }
#ifdef USE_MP4
    } else if(getType()==_T("flac") || getType()==_T("ogg") || getType()==_T("mp4")) { //flac or ogg
#else
    } else if(getType()==_T("flac") || getType()==_T("ogg")) { //flac or ogg
#endif
        MaListe* ml=NULL;
        if(getType()==_T("flac")) ml=mlisteFlac;
        else if(getType()==_T("ogg")) ml=mlisteOgg;
#ifdef USE_MP4
        else if(getType()==_T("mp4")) ml=mlisteMp4;
#endif
        if (ml->GetSelectedItemCount() == 1) { //If only one row is selected
            book->getComboAlbum2()->SetValue(ml->getSelRowText(MaListe::ALBUM_COL,-1));
            book->getComboTitre2()->SetValue(ml->getSelRowText(MaListe::TITLE_COL,-1));
            book->getComboInterprete2()->SetValue(ml->getSelRowText(MaListe::ARTIST_COL,-1));
            book->getComboAnne2()->SetValue(ml->getSelRowText(MaListe::YEAR_COL,-1));
            book->getComboGenre2()->SetValue(ml->getSelRowText(MaListe::GENRE_COL,-1));
            book->getComboPiste2()->SetValue(ml->getSelRowText(MaListe::TRACK_COL,-1));
        } else if(ml->GetSelectedItemCount() != 0) {
            //Inialize combobox
            book->getComboAlbum2()->SetValue(_("<Album>"));
            book->getComboTitre2()->SetValue(_("<Title>"));
            book->getComboInterprete2()->SetValue(_("<Artist>"));
            book->getComboAnne2()->SetValue(_("<Year>"));
            book->getComboPiste2()->SetValue(_("<Track>"));
            book->getComboGenre2()->SetValue(_("<Genre>"));
        }


    }
}

/* ****************************************************************************** */

void MonChoicebook::disableMenus(bool d) {
    if(d) {
        mlisteV1->sendMenuMsg(-1);//disable all items
    } else { //enable only usable item
        if(mlisteV1->GetItemCount()>0) {
            if(mlisteV1->GetSelectedItemCount()>0)
                mlisteV1->sendMenuMsg(0);
            else
                mlisteV1->sendMenuMsg(2);
        }
    }
}
