/**************************************************************************
* Name:      Music_Format.cpp                                             *
* Purpose:   Code for the Music Format class                              *
* Author:    Jackie Team (jackie.team@gmail.com)                          *
* Created:   2009-02-08                                                   *
* Copyright: Jackie Team (http://www.musicorganizer.org)                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pb.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "Music_Format.h"

/*********************************************************************************
 *                                  MusicFormat                                  *
 *********************************************************************************/

/**
 * Static function which format a time (in seconds)
 * into a human readable string
 */
wxString MusicFormat::formatToTime(int length) {
    int heure=0,minute=0,seconde=0;
    wxString duree;
    //First compute hours, minutes and seconds...
    seconde = length % 60;
    if (length >= 60)
        minute = length / 60;
    if(length >= 3600)
        heure = length / 3600;

    //...then build the string
    if(heure != 0)
        duree.Printf(_T("%d:%02d:%02d"),heure,minute,seconde);
    else
        duree.Printf(_T("%d:%02d"),minute,seconde);

    return duree;
}

/*********************************************************************************
 *                                     ID3V1                                     *
 *********************************************************************************/
MusicFormatID3V1::MusicFormatID3V1(wxString f) : MusicFormat() {
    file = new TagLib::MPEG::File(f.fn_str(),true,TagLib::AudioProperties::Average);
}

bool MusicFormatID3V1::exists() {
    return (file->ID3v1Tag() != NULL);
}

wxString MusicFormatID3V1::getArtist() {
    return wxString(file->ID3v1Tag()->artist().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V1::getTitle() {
    return wxString(file->ID3v1Tag()->title().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V1::getAlbum() {
    return wxString(file->ID3v1Tag()->album().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V1::getGenre() {
    return wxString(file->ID3v1Tag()->genre().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V1::getTrack() {
    wxString track;
    track.Printf(_T("%u"),file->ID3v1Tag()->track());
    if (track == _T("0"))
        track = wxEmptyString;
    return track;
}
wxString MusicFormatID3V1::getYear() {
    wxString year;
    year.Printf(_T("%u"),file->ID3v1Tag()->year());
    if (year == _T("0"))
        year = wxEmptyString;
    return year;
}

wxString MusicFormatID3V1::getLength() {
    try {
        if (file->audioProperties() == NULL)
            return _T("?");
        int len = file->audioProperties()->length();
        return formatToTime(len);
    } catch(...) {
        return _T("?");
    }
}

void MusicFormatID3V1::setArtist(wxString artist) {
    file->ID3v1Tag(true)->setArtist(TagLib::String(artist.fn_str()));
}
void MusicFormatID3V1::setTitle(wxString title) {
    file->ID3v1Tag(true)->setTitle(TagLib::String(title.fn_str()));
}
void MusicFormatID3V1::setAlbum(wxString album) {
    file->ID3v1Tag(true)->setAlbum(TagLib::String(album.fn_str()));
}
void MusicFormatID3V1::setGenre(wxString genre) {
    file->ID3v1Tag(true)->setGenre(TagLib::String(genre.fn_str()));
}
void MusicFormatID3V1::setTrack(int track) {
    file->ID3v1Tag(true)->setTrack(track);
}
void MusicFormatID3V1::setYear(int year) {
    file->ID3v1Tag(true)->setYear(year);
}

void MusicFormatID3V1::save() {
    file->save(TagLib::MPEG::File::ID3v1,false);
}

/*********************************************************************************
 *                                     ID3V2                                     *
 *********************************************************************************/

MusicFormatID3V2::MusicFormatID3V2(wxString f) : MusicFormat() {
    file = new TagLib::MPEG::File(f.fn_str(),true,TagLib::AudioProperties::Average);
}

bool MusicFormatID3V2::exists() {
    return (file->ID3v2Tag() != NULL);
}

wxString MusicFormatID3V2::getArtist() {
    return wxString(file->ID3v2Tag()->artist().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V2::getTitle() {
    return wxString(file->ID3v2Tag()->title().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V2::getAlbum() {
    return wxString(file->ID3v2Tag()->album().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V2::getGenre() {
    return wxString(file->ID3v2Tag()->genre().toCString(true),wxConvUTF8);
}
wxString MusicFormatID3V2::getTrack() {
    wxString track;
    track.Printf(_T("%u"),file->ID3v2Tag()->track());
    if (track == _T("0"))
        track = wxEmptyString;
    return track;
}
wxString MusicFormatID3V2::getYear() {
    wxString year;
    year.Printf(_T("%u"),file->ID3v2Tag()->year());
    if (year == _T("0"))
        year = wxEmptyString;
    return year;
}

wxString MusicFormatID3V2::getLength() {
    try {
        if (file->audioProperties() == NULL)
            return _T("?");
        int len = file->audioProperties()->length();
        return formatToTime(len);
    } catch(...) {
        return _T("?");
    }
}

void MusicFormatID3V2::setArtist(wxString artist) {
    file->ID3v2Tag(true)->setArtist(TagLib::String(artist.fn_str()));
}
void MusicFormatID3V2::setTitle(wxString title) {
    file->ID3v2Tag(true)->setTitle(TagLib::String(title.fn_str()));
}
void MusicFormatID3V2::setAlbum(wxString album) {
    file->ID3v2Tag(true)->setAlbum(TagLib::String(album.fn_str()));
}
void MusicFormatID3V2::setGenre(wxString genre) {
    file->ID3v2Tag(true)->setGenre(TagLib::String(genre.fn_str()));
}
void MusicFormatID3V2::setTrack(int track) {
    file->ID3v2Tag(true)->setTrack(track);
}
void MusicFormatID3V2::setYear(int year) {
    file->ID3v2Tag(true)->setYear(year);
}

void MusicFormatID3V2::save() {
    file->save(TagLib::MPEG::File::ID3v2,false);
}

/*********************************************************************************
 *                                      OGG                                      *
 *********************************************************************************/

MusicFormatOGG::MusicFormatOGG(wxString f) : MusicFormat() {
    file = new TagLib::FileRef(f.fn_str());
}

bool MusicFormatOGG::exists() {
    return (!file->isNull() && file->tag());
}

wxString MusicFormatOGG::getArtist() {
    return wxString(file->tag()->artist().toCString(true),wxConvUTF8);
}
wxString MusicFormatOGG::getTitle() {
    return wxString(file->tag()->title().toCString(true),wxConvUTF8);
}
wxString MusicFormatOGG::getAlbum() {
    return wxString(file->tag()->album().toCString(true),wxConvUTF8);
}
wxString MusicFormatOGG::getGenre() {
    return wxString(file->tag()->genre().toCString(true),wxConvUTF8);
}
wxString MusicFormatOGG::getTrack() {
    wxString track;
    track.Printf(_T("%u"),file->tag()->track());
    if (track == _T("0"))
        track = wxEmptyString;
    return track;
}
wxString MusicFormatOGG::getYear() {
    wxString year;
    year.Printf(_T("%u"),file->tag()->year());
    if (year == _T("0"))
        year = wxEmptyString;
    return year;
}

wxString MusicFormatOGG::getLength() {
    try {
        if (file->audioProperties() == NULL)
            return _T("?");
        int len = file->audioProperties()->length();
        return formatToTime(len);
    } catch(...) {
        return _T("?");
    }
}

void MusicFormatOGG::setArtist(wxString artist) {
    file->tag()->setArtist(TagLib::String(artist.fn_str()));
}
void MusicFormatOGG::setTitle(wxString title) {
    file->tag()->setTitle(TagLib::String(title.fn_str()));
}
void MusicFormatOGG::setAlbum(wxString album) {
    file->tag()->setAlbum(TagLib::String(album.fn_str()));
}
void MusicFormatOGG::setGenre(wxString genre) {
    file->tag()->setGenre(TagLib::String(genre.fn_str()));
}
void MusicFormatOGG::setTrack(int track) {
    file->tag()->setTrack(track);
}
void MusicFormatOGG::setYear(int year) {
    file->tag()->setYear(year);
}

void MusicFormatOGG::save() {
    file->save();
}

/*********************************************************************************
 *                                      FLAC                                     *
 *********************************************************************************/

MusicFormatFLAC::MusicFormatFLAC(wxString f) : MusicFormatOGG() {
    file = new TagLib::FileRef(f.fn_str());
}

#ifdef USE_MP4
/*********************************************************************************
 *                                      MP4                                      *
 *********************************************************************************/

MusicFormatMP4::MusicFormatMP4(wxString f) : MusicFormat() {
    file = new TagLib::MP4::File(f.fn_str(),true,TagLib::AudioProperties::Average);
}

bool MusicFormatMP4::exists() {
    return (file->tag());
}

wxString MusicFormatMP4::getArtist() {
    return wxString(file->tag()->artist().toCString(true),wxConvUTF8);
}
wxString MusicFormatMP4::getTitle() {
    return wxString(file->tag()->title().toCString(true),wxConvUTF8);
}
wxString MusicFormatMP4::getAlbum() {
    return wxString(file->tag()->album().toCString(true),wxConvUTF8);
}
wxString MusicFormatMP4::getGenre() {
    return wxString(file->tag()->genre().toCString(true),wxConvUTF8);
}
wxString MusicFormatMP4::getTrack() {
    wxString track;
    track.Printf(_T("%u"),file->tag()->track());
    if (track == _T("0"))
        track = wxEmptyString;
    return track;
}
wxString MusicFormatMP4::getYear() {
    wxString year;
    year.Printf(_T("%u"),file->tag()->year());
    if (year == _T("0"))
        year = wxEmptyString;
    return year;
}

wxString MusicFormatMP4::getLength() {
    try {
        if (file->audioProperties() == NULL)
            return _T("?");
        int len = file->audioProperties()->length();
        return formatToTime(len);
    } catch(...) {
        return _T("?");
    }
}

void MusicFormatMP4::setArtist(wxString artist) {
    file->tag()->setArtist(TagLib::String(artist.fn_str()));
}
void MusicFormatMP4::setTitle(wxString title) {
    file->tag()->setTitle(TagLib::String(title.fn_str()));
}
void MusicFormatMP4::setAlbum(wxString album) {
    file->tag()->setAlbum(TagLib::String(album.fn_str()));
}
void MusicFormatMP4::setGenre(wxString genre) {
    file->tag()->setGenre(TagLib::String(genre.fn_str()));
}
void MusicFormatMP4::setTrack(int track) {
    file->tag()->setTrack(track);
}
void MusicFormatMP4::setYear(int year) {
    file->tag()->setYear(year);
}

void MusicFormatMP4::save() {
    file->save();
}
#endif
